c23456789012345678901234567890123456789012345678901234567890123456789012
      subroutine dpsinit(iunit,lbox,lcap)
c     This procedure initializes PostScript graphics on unit number 
c     iunit, with a box around the drawing area if lbox is .true.

      implicit double precision(a-h,o-y),logical (l),complex (z)

c     Prologue to PS file; definitions.
      if(lcap)then
c        Set bounding box to include caption.
         write( unit = iunit, fmt = 1001)
1001     format(
     $    '%!PS'/
     $    '%%BoundingBox: 36 36 576 756')
      else
c        Set bounding box to not include caption.
         write( unit = iunit, fmt = 1002)
1002     format(
     $    '%!PS'/
     $    '%%BoundingBox: 36 216 576 756')
      end if

      write( unit = iunit, fmt = 1003)
1003  format(
     $ 'save'/
     $ '/l {lineto} bind def'/ '/n {newpath} bind def'/
     $ '/m {moveto} bind def'/
     $ '/t {stroke} bind def'/
     $ '/g {setgray} bind def'/
     $ '/f {fill} bind def'/
     $ '/c {closepath} bind def'/
     $ '/h {show} bind def'/
     $ '/r {rlineto} bind def'/
     $ '/rm {rmoveto} bind def'
     $ )

c     Origin and scaling transformations, and line parameters.
      write( unit = iunit, fmt = 1004)
1004  format(
     $ '36 216 translate'/
     $ '/SF 540 def SF SF scale'/
     $ '1 setlinecap'/
     $ '1 setlinejoin'
     $ )

c     Draw a box around drawing area.
      if(lbox) write( unit = iunit, fmt = 1005)
1005  format(
     $ '0.002 setlinewidth'/
     $ '0 0 m'/
     $ '0 1 l'/
     $ '1 1 l'/
     $ '1 0 l'/
     $ '0 0 l'/
     $ 't'
     $ )

c     Mark end of dpsinit.f output.
      write( unit = iunit, fmt = 1006)
1006  format('%End of dpsinit.f output')

      return
      end
      subroutine dpsterm(iunit)
c     This procedure terminates PS graphics on unit number iunit.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      write( unit = iunit, fmt = 1001)
1001  format(
     $ '%Beginning of dpsterm.f output'/
     $ 'restore'/
     $ 'showpage'/
     $ '%End of dpsterm.f output'
     $ )
      return
      end
      subroutine dpsnumfile(num,ipos,name,iunit)
c     This file keeps track of the files used for output;
c     it assumes that the name of the file is input in the
c     character variable 'name' and that the number 'num' is to be imbedded 
c     in 'name' in positions ipos through ipos+3, so the filename
c     has the form abcabcabc1234.defdfef where the number begins
c     in position ipos. The function of this routine is to 
c     put 'num' into 'name', increment it by one,
c     and open a file named 'name' on Fortran unit number iunit.
c     Typical usage is to use a series of individual PS pages
c     in files fig.1000.ps, fig.1001.ps,...  in which case ipos = 5.
c     The reason for starting at 1000
c     instead of 0000 is to avoid leading zero usage conflicts in shell scripts.
c     If iunit is negative, then output is not desired and the unix
c     file /dev/null is opened on unit number min(abs(iunit),99).

      implicit double precision (a-h,o-y), complex(z)
      character*(*) name
      if(iunit.gt.0.and.iunit.lt.100)then
c        Unpack num into four characters.
         i3 = num/1000
         i2 = num/100 -10  *i3
         i1 = num/10  -100 *i3-10 *i2
         i0 = num/1   -1000*i3-100*i2-10*i1
         name(ipos:ipos+3) = 
     $  char(48+i3)//char(48+i2)//char(48+i1)//char(48+i0)
         num = num+1
c        Open table file on unit iunit.
         open(unit = iunit,form = 'formatted',status = 'unknown',
     $     file = name,access = 'sequential')
      else
	 iunit = min(abs(iunit),99)
c        Open unix trash bin on unit |iunit|.
         open(unit = iunit,form = 'formatted',status = 'unknown',
     $     file = '/dev/null',access = 'sequential')
      end if
      return
      end
      subroutine dpscurve(iunit,n,x,y,xl,xr,yl,yr,thickness,gray)
c     This routine draws a curve of n points given in x and y on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white. 

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) thickness,gray
1001  format(' gsave ',f6.4,' setlinewidth ',f6.4,' g')

      i = 1
      xx = (x(i)-xl)*xlen
      yy = (y(i)-yl)*ylen
      write( unit = iunit, fmt = 1002) xx,yy
1002  format(f6.4,1x,f6.4,' m')

      do 10 i = 2,n
         xx = (x(i)-xl)*xlen
         yy = (y(i)-yl)*ylen
         write( unit = iunit, fmt = 1003) xx,yy
10    continue
1003  format(f6.4,1x,f6.4,' l')

c     Stroke curve.
      write( unit = iunit, fmt = 1004)
1004  format('t')

      write( unit = iunit, fmt = 1005)
1005  format(' grestore')

      return
      end

      subroutine dpsdot(iunit,n,x,y,xl,xr,yl,yr,radius,gray)
c     This routine draws dots of radius 'radius' at n points given
c     in array x and y, in grayscale color 'gray (1 = white,0 = black).
c     The drawing area is [0,1]^2 and x and y are scaled by xr,xl,yr,yl.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) gray,radius
1001  format('gsave ',f6.4,' g'/
     $ '/d { ',f6.4,' 0 360 arc f} bind def'/
     $ 'n'
     $ )

      do 10 i = 1,n
         xx = (x(i)-xl)*xlen
         yy = (y(i)-yl)*ylen
         write( unit = iunit, fmt = 1002) xx,yy
10    continue
1002  format(f6.4,1x,f6.4,' d')
      write( unit = iunit, fmt = 1003)
1003  format(' t grestore')

      return
      end

      subroutine dpsvdot(iunit,n,x,y,xl,xr,yl,yr,radius,gray)
c     This routine draws dots of variable radii 'radius' at n points given
c     in array x and y, in grayscale color 'gray (1 = white,0 = black).
c     The drawing area is [0,1]^2 and x and y are scaled by xr,xl,yr,yl.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1),radius(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) gray
1001  format('gsave ',f6.4,' g'/
     $ '/d {0 360 arc f} bind def'/
     $ 'n'
     $ )

c     Draw individual dots.
      radscal = max(xlen,ylen)
      do 10 i = 1,n
         xx = (x(i)-xl)*xlen
         yy = (y(i)-yl)*ylen
         rr = radius(i)*radscal
         write( unit = iunit, fmt = 1002) xx,yy,radius(i)
10    continue
1002  format(f6.4,1x,f6.4,1x,f6.4,' d')
c     Finish drawing.
      write( unit = iunit, fmt = 1003)
1003  format(' t grestore')

      return
      end

      subroutine dpsseg(iunit,x1,y1,x2,y2,xl,xr,yl,yr)
c     This routine draws a segment of 2 points given in x and y on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white.  These parameters are set in dpsseginit.
c     Segments lying completely outside the drawing area are skipped.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      xlen = 1.0/(xr-xl)
      if(xlen.le.0.0)return
      ylen = 1.0/(yr-yl)
      if(ylen.le.0.0)return
c     Skip segments outside drawing area.
      if(max(x1,x2).lt.xl)return
      if(min(x1,x2).gt.xr)return
      if(max(y1,y2).lt.yl)return
      if(min(y1,y2).gt.yr)return

      xx = (x1-xl)*xlen
      yy = (y1-yl)*ylen
      write( unit = iunit, fmt = 1002) xx,yy
1002  format(f6.4,1x,f6.4,' m')

      xx = (x2-xl)*xlen
      yy = (y2-yl)*ylen
      write( unit = iunit, fmt = 1003) xx,yy
1003  format(f6.4,1x,f6.4,' l')

c     Stroke curve.
      write( unit = iunit, fmt = 1004)
1004  format('t')

      return
      end
      subroutine dpsgraphinit(iunit,xl,xr,xa,yl,yr,ya,
     $ th,grayl,nxt,nyt,xt,yt,cx,cy,ipsize)
c     This routine initializes a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     Tick marks are drawn if nxt (resp. nyt) is > 0,
c     there are nxt of them. They are labeled according to xl and xr.
c     The axes with tick marks and labels are drawn at locations xa and ya.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      character cx*(*),cy*(*)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) 
     >   (xa-xl)*xlen,(ya-yl)*ylen,th,ipsize
1001  format('%Begin dpsgraphinit.f output'/
     $ 'gsave /XM 0.10 def /YM 0.10 def'/
     $ ' /XS 0.85 def /YS 0.85 def '/
     $ ' /XA ',f6.4,' def /YA ',f6.4,' def '/
     $ 'XM YM translate XS YS scale'/
     $ f6.4,' setlinewidth'/
     $ '/Helvetica findfont /PS ',i4,
     > ' SF div def PS scalefont setfont'/
     $ '/LS 1.25 PS mul def ')
c     Draw box around graph.
      write( unit = iunit, fmt = 1020)
1020  format('0 0 m 0 1 r 1 0 r 0 -1 r closepath t')
      if(xl.le.0.0.and.xr.ge.0.0)then
c     Draw x-axis crossing at x = xa.
      write( unit = iunit, fmt = 1002) 
1002  format(' XA 0 m 0 1 r t')
      end if
      if(yl.le.0.0.and.yr.ge.0.0)then
c     Draw y-axis crossing at y = 0.
      write( unit = iunit, fmt = 1003)
1003  format(' 0 YA m 1 0 r t')
      end if

c     Tick marks on x-axis.
      if(nxt.gt.0)then
      write( unit = iunit, fmt = 1004) xt
1004  format('/txl {dup YA m 0 ',f12.4,
     $ ' r t YA LS sub m dup stringwidth pop ',
     $ '2 div neg 0 rm h } def')
      do 100 ixt = 0,nxt
ccc      do 100 ixt = 1,nxt
      xx = dble(ixt)/dble(nxt)
c           write( unit = iunit, fmt = 1005) xx*(xr-xl)+xl,xx
      write( unit = iunit, fmt = 1005) nint(xx*(xr-xl)+xl),xx
100   continue
c     1005  format('(',f9.2,') ',g10.4,' txl')
1005  format('(',i2,') ',g10.4,' txl')
      end if

c     Caption for x-axis.
      write( unit = iunit, fmt = 1007) cx
1007  format('0.5 YM 0.66 mul neg  m (',a,')'/
     $ ' dup stringwidth pop 2 div neg PS 2 div neg rm h')

c     Tick marks on y-axis.
      if(nyt.gt.0)then
      write( unit = iunit, fmt = 1008) yt
1008  format('/tyl { dup XA exch m ',f6.4,' 0 r t'/
     $ 'XA exch m dup stringwidth pop neg PS 2 div neg rm h } def')
ccc      do 200 iyt = 0,nyt
      do 200 iyt = 1,nyt
      yy = dble(iyt)/dble(nyt)
c      write( unit = iunit, fmt = 1009) yy*(yr-yl)+yl,yy
      write( unit = iunit, fmt = 1009) nint(yy*(yr-yl)+yl),yy
200   continue
c1009  format('(',f6.1,' ) ',f6.4,' tyl')
1009  format('(',i2,' ) ',f6.4,' tyl')
      end if

c     Caption for y-axis.
      write( unit = iunit, fmt = 1011) cy
1011  format(' gsave 90 rotate 0.5 XM 0.66 mul m (',a,')'/
     $ ' dup stringwidth pop 2 div neg PS 1.5 mul rm h grestore'/
     $ ' %End dpsgraphinit.f output')

      return
      end
      subroutine dpsgraphterm(iunit)
c     This routine terminates a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)

      write( unit = iunit, fmt = 1013)
1013  format('%Begin dpsgraphterm.f output'/
     $  'grestore'/
     $  '%End dpsgraphterm.f output')
      
      return
      end

      subroutine dpsgraph(iunit,n,x,y,xl,xr,yl,yr,
     $ radius,dotgray,ndash,dash,th,grayl)
c     This routine draws a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     There are n data points given.
c     Tick marks are drawn if nxt (resp. nyt) is > 0,
c     there are nxt of them. They are labeled according to xl and xr.
c     The axes with tick marks and labels are drawn at locations xa and ya.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.
c     The line is drawn with dash pattern specified by ndash and dash
c     according to the postscript usage; ndash <11 is the array length,
c     the elements of dash specify the dash pattern.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1),dash(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      if(radius.gt.0.0)then
      call dpsdot(iunit,n,x,y,xl,xr,yl,yr,radius,dotgray)
      end if

      if(th.gt.0.0)then
      if(ndash.le.0)then
         write( unit = iunit, fmt = 1001)
1001     format('[] 0 setdash')
      else
         write( unit = iunit, fmt = 1002)
         write( unit = iunit, fmt = 1003) (dash(i),i = 1,ndash)
         write( unit = iunit, fmt = 1004)
1002     format('[')
1003     format((1x,f10.4))
1004     format('] 0 setdash ')
      end if
      call dpsseginit(iunit,xl,xr,yl,yr,th,grayl)
      do 300 i = 1,n-1
      call dpsseg(iunit,x(i),y(i),x(i+1),y(i+1),xl,xr,yl,yr)
300   continue
      call dpssegterm(iunit)
      end if

      return
      end

      subroutine dpsgraphs( iunit, n, m, x, nq, y, xl, xr, yl, yr, 
     >   ndash, dash, nxt, th, grayl )
c     This routine draws a graph of nq variables y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     There are n data points given and y data at each in y(1:nq, * ) .
c     Tick marks are drawn at min( n, nxt ) data points on the x-axis.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.
c     The line is drawn with dash pattern specified by ndash and dash
c     according to the postscript usage; ndash <11 is the array length,
c     the elements of dash specify the dash pattern.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(nq,1),dash(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)

c     Number of tick marks on x-axis.
      nt = min( n, nxt )
c     Frequency of tick marks on x-axis.
      mt = max( 1, n / max( 1, nt ) )

c     Number of points to plot.
      np = min( n, m )
c     Frequency of points to plot.
      mp = max( 1, n / np )

      if(xlen.le.0.0.or.ylen.le.0.0)return

      if(th.gt.0.0)then

      if(ndash.le.0)then
         write( unit = iunit, fmt = 1001)
1001     format('[] 0 setdash')
      else
         write( unit = iunit, fmt = 1002)
         write( unit = iunit, fmt = 1003) (dash(i),i = 1,ndash)
         write( unit = iunit, fmt = 1004)
1002     format('[')
1003     format((1x,f10.4))
1004     format('] 0 setdash ')
      end if

      call dpsseginit(iunit,xl,xr,yl,yr,th,grayl)
      do iq = 1, nq
         do i = 1, n - 1, mp
            xb = x( i )
            yb = y( iq, i ) 
            ie = min( n, i + mp )
            xe = x( ie )
            ye = y( iq, ie )
            call dpsseg( iunit, xb, yb, xe, ye, xl, xr, yl, yr )
         end do
      end do

c     Draw ticks vertically at location x(i), covering 2% of the vertical span.
      do i = 1, n, mt
         call dpsseg( iunit, x( i ), yl, x( i ), yl + 
     >      0.02 * ( yr - yl ), xl, xr, yl, yr )
      end do

      call dpssegterm(iunit)
      end if

      return
      end

      subroutine dpsgraphletter( iunit, n, x, y, xl, xr, yl, yr,
     >      c, ndash, dash, th, grayl )
c     This routine letterplots a graph of the variable y against the 
c     variable x, assuming xl < x < xr and yl < y < yr.
c     There are n data points given.
c     Tick marks are drawn if nxt (resp. nyt) is > 0,
c     there are nxt of them. They are labeled according to xl and xr.
c     The axes with tick marks and labels are drawn at locations xa and ya.
c     The character variables cx and cy store x and y axis captions,
c     and ipsize is the point size for all characters.
c     Procedure dpsinit should be called before this routine.
c     iunit is the output file unit number.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      dimension x(1),y(1),dash(1)
      character*1 c(1)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      if(th.gt.0.0)then
      if(ndash.le.0)then
         write( unit = iunit, fmt = 1001)
1001     format('[] 0 setdash')
      else
         write( unit = iunit, fmt = 1002)
         write( unit = iunit, fmt = 1003) (dash(i),i = 1,ndash)
         write( unit = iunit, fmt = 1004)
1002     format('[')
1003     format((1x,f10.4))
1004     format('] 0 setdash ')
      end if
      call dpsseginit(iunit,xl,xr,yl,yr,th,grayl)
      do 300 i = 1,n-1
      call dpsseg( iunit, x(i), y(i), x(i+1), y(i+1), xl, xr, yl, yr)
300   continue
      call dpssegterm(iunit)
      end if

      do i = 1, n
 
         xx = ( x( i ) - xl ) * xlen
         yy = ( y( i ) - yl ) * ylen

         write( unit = iunit, fmt = 1012) xx, yy, c( i )
1012     format(f6.4, 1x ,f6.4, ' m (', a1, ') h' )

      end do

      return
      end

      subroutine dpsseginit(iunit,xl,xr,yl,yr,thickness,gray)
c     This routine initializes segment drawing routine dpsseg on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white. 

      implicit double precision(a-h,o-y),logical (l),complex (z)
      xlen = 1.0/(xr-xl)
      ylen = 1.0/(yr-yl)
      if(xlen.le.0.0.or.ylen.le.0.0)return

      write( unit = iunit, fmt = 1001) thickness,gray
1001  format(' gsave ',f6.4,' setlinewidth ',f6.4,' g')

      return
      end
      subroutine dpssegterm(iunit)
c     This routine terminates segment drawing routine dpsseg on
c     unit number iunit, scaled by xr,xl,yr,yl. The drawing surface
c     is [0,1]^2 and set by dpsinit. The line thickness is determined
c     by thickness and the grayscale color of the line by gray: 0 is 
c     black, 1 white. 

      implicit double precision(a-h,o-y),logical (l),complex (z)
      write( unit = iunit, fmt = 1005)
1005  format(' grestore')
      return
      end
      subroutine dpscapinit(iunit,ipsize,lbox)
c     This routine initializes caption drawing in the lower quarter
c     of the page. If lbox is true, a box is drawn around it.
c     Routine dpsinit should be called before this routine,
c     and dpsterm should be called to complete the file;
c     no routine is necessary for terminating the caption.
c     Parameter ipsize is the point size for drawing the caption.
c     Parameter iunit is the unit number on which the figure is written.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      if(lbox) write( unit = iunit, fmt = 1001)
1001  format(
     $ '%Begin dpscapinit.f output'/
     $ '%Box around caption area'/
     $ '0 0 m'/
     $ '1 0 l'/
     $ '1 -.333333 l'/
     $ '0 -.333333 l'/
     $ '0 0 l'/
     $ 't'
     $ )

c     The following lines choose a font (Helvetica), scale it
c     to point size PS = ipsize, and set the line spacing LS,
c     left margin LM and caption line position CL.
c     The point size is divided by the scale factor SF set by dpsinit,
c     and the other parameters are defined in terms of PS.
c     We then define a PS procedure capline which writes a string
c     and then moves the current point down one line spacing.
      write( unit = iunit, fmt = 1002) ipsize
1002  format(
     $ '%Drawing captions.'/
     $ '%Font used must be scaled appropriately.'/
     $ '/Helvetica findfont '/
     $ '/PS ',i3,' SF div def'/
     $ 'PS scalefont setfont'/
     $ '/LS 1.25 PS mul def'/
     $ '/LM  LS def'/
     $ '/CL  LS neg def'/
     $ '/capline {LM CL m h /CL CL LS sub def} def'/
     $ '%End dpscapinit.f output'
     $ )

      return
      end
      subroutine dpscapline(iunit,line)
c     This routine writes one line of the caption, in the lower
c     quarter of the drawing area. It takes a character variable
c     'line' of length less than 50 characters; later I should 
c     do something a little more general.
c     iunit is the unit number on which the PS file is written.
c     Procedure dpscapinit should be called first.

      implicit double precision(a-h,o-y),logical (l),complex (z)
      character*(*) line
      write( unit = iunit, fmt = 1001) line
1001  format(1x,'(',a,') capline')
      return
      end
