c        1         2         3         4         5         6          72
      subroutine zeta( nz, zs, zz, zd, cc, nl, np )
c     Evaluate vector of values zz = zeta( zs, c ) and log derivatives 
c     zd = zeta'/zeta where c is parameter
c     in map, s is vector of powers in zeta sum, nl is number of levels
c     and np number of basis functions. Should give close to zero for s 
c     the dimension associated with c: 
c     zeta( 1, 0.31850809575800, z, -20, 4, 1 ) 
c       gives z = 2.7859e-10 +i* -1.0229e-22
c     and
c     zeta( 1, 0.48479829443816, -5, z, 4, 1 )
c       gives z = 1.2824e-06 +i* -1.1619e-20
c     while with higher accuracy
c     zeta( 1, 0.31850809575800, -20, z, 8, 5 )
c     	gives z = -1.1571e-14 +i* -3.3773e-17
c     and
c     zeta( 1, 0.48479829443816, -5, z, 8, 5 )
c    	gives z = -6.5302e-16 +i* -2.7861e-17
c     

      implicit double precision (a-h,o-y)
      implicit double complex (z)
      parameter( pi = 3.1415926535898d0 )

c     Max number of levels, intervals and basis functions per interval.
      parameter( ml = 8, mi = 2**ml, mp = 5, mip = mi * mp )

c     Order of accuracy for numerical quadrature and number of pts/wts.
c     Tenth order quadrature formula on unit disk from AS p. 892.
      parameter( nq = 21 )

c     Quadrature points and weights.
      dimension zq( nq ), wq( nq )

      data zq /
     > ( 0.0d0, 0.0d0 ),
     > ( 0.4820621466885461, 0.3502386507107874 ),
     > ( 0.1841313553452870, 0.5666980410239565 ),
     > ( -0.1841313553452870, 0.5666980410239566 ),
     > ( -0.4820621466885460, 0.3502386507107875 ),
     > ( -0.5958615826865181, 0.0d0 ),
     > ( -0.4820621466885461, -0.3502386507107874 ),
     > ( -0.1841313553452871, -0.5666980410239565 ),
     > ( 0.1841313553452869, -0.5666980410239566 ),
     > ( 0.4820621466885460, -0.3502386507107875 ),
     > ( 0.5958615826865181, 0.0d0 ),
     > ( 0.7436573695963749, 0.5402987052763671 ),
     > ( 0.2840518392014725, 0.8742216692147242 ),
     > ( -0.2840518392014724, 0.8742216692147243 ),
     > ( -0.7436573695963747, 0.5402987052763673 ),
     > ( -0.9192110607898046, 0.0d0 ),
     > ( -0.7436573695963749, -0.5402987052763670 ),
     > ( -0.2840518392014726, -0.8742216692147242 ),
     > ( 0.2840518392014723, -0.8742216692147243 ),
     > ( 0.7436573695963747, -0.5402987052763674 ),
     > ( 0.9192110607898046, 0.0d0 ) 
     > /
      data wq /
     > 0.3490658503988659,
     > 10 * 0.1610021706622441,
     > 10 * 0.1182505096568486 
     > /

c     Input/output points and values---must be at least nz long.
      dimension zs( * ), zz( * ), zd( * )

c     Interval endpoints and indices.
      dimension emin( mi ), emax( mi ), indices( mi, ml )
      dimension tmp( 2 * mi ), itmp( mi, ml )

c     Matrix of operator and small block matrix.
      dimension zma( mip, mip ), zta( mp, mp )
      dimension zmd( mip, mip ), ztd( mp, mp )
      dimension ipiv( mip )
c     Temporary arrays.
      dimension center( mi ), radius( mi ), sg( ml )
      dimension zqi( nq ), zik( mp ), zbjl( mp ), sqi( mp )

c     Integer matrix to show block structure.
      dimension istruct( mi, mi )

c      write( 6, * ) ' Inside zeta nz, cc, nl, np = ', nz, cc, nl, np

c     Test input parameters so arrays don't overflow.
      if( nl .gt. ml ) then
         write( 6, * ) ' nl = ', nl, ' > ml = ', ml, ' error return'
      end if
      if( np .gt. mp ) then
         write( 6, * ) ' np = ', np, ' > mp = ', mp, ' error return'
      end if
c     Here we should build a quadrature formula that gives desired 
c     accuracy depending on the physical parameters c and s. In the 
c     meantime, we use tenth order quadrature formula on unit disk 
c     from AS p. 892.
c      zq( 1 ) = dcmplx( 0.0d0, 0.0d0 )
c      wq( 1 ) = pi * 1.0d0 / 9.0d0
c      ibeg = 1
c      ss = dsqrt( 0.10d0 * ( 6.0d0 - dsqrt( 6.0d0 ) ) )
c      ww = pi * ( 16.00d0 + dsqrt( 6.00d0 ) ) / 360.0d0
c      do iq = 1, 10
c         tq = 0.20d0 * pi * dble( iq )
c         zq( ibeg + iq ) = ss * dcmplx( cos( tq ), sin( tq ) )
c         wq( ibeg + iq ) = ww
c      end do
c
c      ibeg = 11
c      ss = dsqrt( 0.10d0 * ( 6.0d0 + dsqrt( 6.0d0 ) ) )
c      ww = pi * ( 16.0d0 - dsqrt( 6.0d0 ) ) / 360.0d0
c      do iq = 1, 10
c         tq = 0.20d0 * pi * dble( iq )
c         zq( ibeg + iq ) = ss * dcmplx( cos( tq ), sin( tq ) )
c         wq( ibeg + iq ) = ww
c      end do
c
c     Initialize and set arrays to maximum sizes required.
 
      ni = 2
      dd = 0.50d0 * ( 1.00d0 + dsqrt( 1.0d0 - 4.00d0 * cc ) )
c      write( 6, * ) ' cc, dd = ', cc, dd

      e1 = dsqrt( -cc - dd ) - 0.250d0
      e2 = -dd - 0.250d0
      emin( 1 ) = min( e1, e2 )
      emin( 2 ) = max( e1, e2 )
      e1 = dd + 0.250d0
      e2 = -dsqrt(-cc - dd ) + 0.250d0
      emax( 1 ) = min( e1, e2 )
      emax( 2 ) = max( e1, e2 )
      indices( 1, 1 ) = 1
      indices( 2, 1 ) = 2
c      write( 6, * ) ' emin = ', ( emin( ii ), ii = 1, ni )
c      write( 6, * ) ' emax = ', ( emax( ii ), ii = 1, ni )
 
c     Iterate through levels il, mapping and sorting interval endpts.
      do il = 2, nl
c         write( 6, * ) ' Intervals on level il = ', il, ' of nl = ', nl
  
         ni2 = 2 * ni

         do ii = 1, ni
            ji = ii + ni
            tmp( ii ) = -1.0d0 * dsqrt( emin( ii ) - cc )
            tmp( ji ) = dsqrt( emin( ii ) - cc )
         end do
         do ii = 1, ni2
            emin( ii ) = tmp( ii )
         end do
 
         do ii = 1, ni
            ji = ii + ni
            tmp( ii ) = -1.0d0 * dsqrt( emax( ii ) - cc )
            tmp( ji ) = dsqrt( emax( ii ) - cc )
         end do
         do ii = 1, ni2 
            emax( ii ) = tmp( ii )
         end do
    
         do jl = 1, il - 1
            do ii = 1, ni
               ji = ii + ni
               itmp( ii, jl + 1 ) = indices( ii, jl )
               itmp( ji, jl + 1 ) = indices( ii, jl )
            end do
         end do
         do ii = 1, ni
            ji = ii + ni
            itmp( ii, 1 ) = 1
            itmp( ji, 1 ) = 2
         end do
         do jl = 1, il
            do ii = 1, ni2
               indices( ii, jl ) = itmp( ii, jl )
            end do
         end do

         ni = ni2
         nip = ni * np

c        Reorder emin and emax as necessary so emin(ii) < emax(ii) foreach ii.
         do ii = 1, ni
           if( emin( ii ) .gt. emax( ii ) ) then
             swap = emax( ii )
             emax( ii ) = emin( ii )
             emin( ii ) = swap
           end if
         end do
c         write( 6, * ) ' emin = ', ( emin( ii ), ii = 1, ni )
c         write( 6, * ) ' emax = ', ( emax( ii ), ii = 1, ni )

c        End loop through levels do il = 2, nl
      end do
c      write( 6, * ) ' Intervals completed nl = ', nl

c     Now we have the endpts in pairs: form matrix of transfer operator
c     in basis of complex powers centered and scaled to each interval.

      do ii = 1, ni
         radius( ii ) = 0.50d0 * ( emax( ii ) - emin( ii ) )
         center( ii ) = emin( ii ) + radius( ii )
      end do
c      write( 6, * ) ' emin = ', ( emin( ii ), ii = 1, ni )
c      write( 6, * ) ' emax = ', ( emax( ii ), ii = 1, ni )
c      write( 6, * ) ' radius = ', ( radius( ii ), ii = 1, ni )
c      write( 6, * ) ' center = ', ( center( ii ), ii = 1, ni )

c     Loop through input values of s, evaluating zeta and zd at each point zt.
      do iz = 1, nz
         zt = zs( iz )
c         write( 6, * ) ' At iz = ', iz, ' with zt = ', zt

c        Initialize big matrices zma and zmd to complex zeroes.
         nip = ni * np
         do i = 1, nip
            do j = 1, nip
               zma( i, j ) = dcmplx( 0.0d0, 0.0d0 )
               zmd( i, j ) = dcmplx( 0.0d0, 0.0d0 )
            end do
         end do
         do ii = 1, ni
            do ji = 1, ni
               istruct( ii, ji ) = 0
            end do
         end do
  
c        Loop through rows ii, building two nonzero blocks per row.
         do ii = 1, ni

c           Each nonzero block in column ji corresponds to ib =1 or 2.
            do ib = 1, 2

c              Compute index ji of nonzero block for index ib.
               sg( 1 ) = ib
               do il = 2, nl
                  sg( il )  = indices( ii, il - 1 )
               end do
               ji = 1
               m = 1
               do il = nl, 1, -1
                  ji = ji + ( sg( il ) - 1 ) * m
                  m = 2 * m
               end do

               istruct( ii, ji ) = ib

c              Form np x np block (ii, ji) by integrating basis functions against 
c              transformed basis functions times power of square root over circle 
c              D_ii.  
c              a = block( ii, ji, ib, np, nl, center, radius, indices, cc, zt )
c              Build nonzero PxP block at location ( ii, ji ) in block matrix, by 
c              integrating f_ik* vs L_b f_jl over circle D_ii for basis functions 
c              1 <= k,l <= np on circles:
c        
c              f_ik(z) = ( 1/r_ii) dsqrt( (k+1) / pi ) ( ( z - c_ii ) / r_ii )**k
c        
c              Here f_jl is transformed by L_ib to yield
c           
c              L_b f_jl(z) = [ g_ib'(z) ]**zt f_jl( g_ib(z) ).
c           
c              The square brackets indicate that we always take the positive sign so 
c           
c              [ g_b'(z) ] = 1 / ( 2 * dsqrt( z - cc ) ).
        
               ri = radius( ii )
               ci = center( ii )
               rj = radius( ji )
               cj = center( ji )
     
c              Shift and scale quadrature points to D_i
               do iq = 1, nq
                  zqi( iq ) = ci + ri * zq( iq )
               end do
       
c              Integrate f_ik* vs L_b f_jl over D_i and accumulate in zta( ip, jp )
      
c              Initialize to zeroes first.
               do ip = 1, np
                  do jp = 1, np
                     zta( ip, jp ) = dcmplx( 0.0d0, 0.0d0 )
                     ztd( ip, jp ) = dcmplx( 0.0d0, 0.0d0 )
                  end do
               end do
       
               do iq = 1, nq
                  z = zqi( iq )
                  w = wq( iq )
       
c                 This wastes arithmetic since we are undoing what we just did.
c                 zi = dconjg( z - ci ) / ri
c                 This does less arithmetic and produces different results.
                  zi = dconjg( zq( iq ) )
       
c                 Matlab code from zlogd.m
c                 gs = (4*( z - c ))^(-s/2);
c                 wgs = w * gs;
c                 % For the derivative wrt s to be stored in ztd.
c                 wlg = -0.5 * wgs * log(4*( z - c ));
 
                  zgs = ( 4.0d0 * ( z - cc ) )**( -0.50d0 * zt )
                  zwgs = w * zgs
                  zwlg = -0.50d0 * zwgs * log( 4.0d0 * ( z - cc ) );
c                  write( 6, * ) ' zgs, zwgs, zwlg = ', zgs, zwgs, zwlg
       
                  zb =(-1)**ib * sqrt( z - cc )
                  zbj = dconjg( zb - cj ) / rj

c                  write( 6, * ) ' ii, ji, iq, zgs, zwgs, zb, zbj = ', 
c     >               ii, ji, iq, zgs, zwgs, zb, zbj
       
c                 Tabulate powers. 
c                 Later we should just do this once for all blocks to save time.
                  zik( 1 ) = dcmplx( 1.0d0, 0.0d0 )
                  do k = 1, np - 1
                     zik( k + 1 )  = zik( k ) * zi
                  end do
c                  write( 6, * ) ' zik = ', ( zik( k ), k = 1, np )
    
                  zbjl( 1 ) = dcmplx( 1.0d0, 0.0d0 )
                  do k = 1, np - 1
                     zbjl( k + 1 )  = zbjl( k ) * zbj
                  end do
c                  write( 6, * ) ' zbjl = ', ( zbjl( k ), k = 1, np )
       
c                 a = a + wgs * zik * zbjl'
c                 d = d + wlg * zik * zbjl';

                  do ip = 1, np
                     do jp = 1, np
                        zterm = zik( ip ) * dconjg( zbjl( jp ) )
                        zta( ip, jp ) = zta( ip, jp ) + zwgs * zterm
                        ztd( ip, jp ) = ztd( ip, jp ) + zwlg * zterm
                     end do
                  end do

c                  write( 6, * ) ' after update zta = ', 
c     >            ( ( zta( i, j ), j =1, np ), i = 1, np )
   
c                 End of do iq = 1, nq over quadrature points.
               end do

c               write( 6, * ) ' before scaling ii, ji, a = ', ii, ji
c               do i = 1, np
c                  write( 6, * ) ( zta( i, j ), j = 1, np )
c               end do
    
c              Final scaling by factors that don't depend on anything else.
               do ip = 1, np
                  do jp = 1, np
                     zta( ip, jp ) = ri * zta( ip, jp ) / ( rj * pi )
                     ztd( ip, jp ) = ri * ztd( ip, jp ) / ( rj * pi )
                  end do
               end do
     
               do ip = 1, np
                  sqi( ip ) = dsqrt( dble( ip ) )
               end do
               do ip = 1, np
                  do jp = 1, np
                     zta( ip, jp ) = sqi( ip ) * zta( ip, jp )
                     ztd( ip, jp ) = sqi( ip ) * ztd( ip, jp )
                  end do
               end do
               do jp = 1, np
                  do ip = 1, np
                     zta( ip, jp ) = sqi( jp ) * zta( ip, jp )
                     ztd( ip, jp ) = sqi( jp ) * ztd( ip, jp )
                  end do
               end do
    
c              Store blocks in zma and zmd in location ( ii, ji ).
c Lm( ( ii - 1 ) * np + 1 : ii * np, ( ji - 1 ) * np + 1 : ji * np ) = a
    
               do ip = 1, np
                  iiip = ( ii - 1 ) * np + ip
                  do jp = 1, np
                     jijp = ( ji - 1 ) * np + jp
                     zma( iiip, jijp ) = -zta( ip, jp )
                     zmd( iiip, jijp ) = -ztd( ip, jp )
                  end do
               end do

c              End do ib = 1, 2 loop through signs of g operator.
            end do
c           End do ii = 1, ni loop through blocks of big matrix.
         end do

c        Replace zma by I + zma --- note have already changed sign in zma above.
         do i = 1, nip
            zma( i, i ) = dcmplx( 1.0d0, 0.0d0 ) + zma( i, i )
         end do
 
         if( nip .le. 8 ) then
c           Print matrices.

            write( 6, * ) ' initial zma = '
            do i = 1, nip
               write( 6, * ) ( zma( i, j ), j = 1, nip )
            end do

            write( 6, * ) ' initial zmd = '
            do i = 1, nip
               write( 6, * ) ( zmd( i, j ), j = 1, nip )
            end do

         end if

c        Calculate zeta function det( I - Lm ).
c        zz( iz ) = det( zma )
c        Calculate logarithmic derivative of zeta function.
c        zd( iz ) = trace( zma \ zmd )

c        Overwrite zma with its PLU factors (U on diagonal) and zmd with
c        zma^-1 * zmd, modulo permutation P stored in ipiv.
         call zgesv( nip, nip, zma, mip, ipiv, zmd, mip, info )
c         write( 6, * ) ' info = ', info

c        Extract sign of determinant from permutation: row i swapped with ipiv(i).
         idet = 1
         do i = 1, nip
            if( ipiv( i ) .ne. i ) idet = -idet
         end do
c        Extract determinant from U factor (ignore permutation).
         zdet = dcmplx( idet, 0.0d0 )
         do i = 1, nip
            zdet = zdet * zma( i, i )
         end do

c        Extract trace of zmd which is log derivative of zeta.
         ztrace = dcmplx( 0.0d0, 0.0d0 )
         do i = 1, nip
            ztrace = ztrace + zmd( i, i )
         end do

c        Store zeta and zlogd in appropriate array elements.
         zz( iz ) = zdet
         zd( iz ) = ztrace

         if( nip .le. 8 ) then
c           Print matrices.

            write( 6, * ) ' factorized zma = '
            do i = 1, nip
               write( 6, * ) ( zma( i, j ), j = 1, nip )
            end do

            write( 6, * ) ' solution zmd = '
            do i = 1, nip
               write( 6, * ) ( zmd( i, j ), j = 1, nip )
            end do
   
         end if


c        End of do iz = 1, nz loop over points of evaluation.
      end do

      if( ni .le. 16 ) then
c        Print block structure matrix.
         do ii = 1, ni
1000        format( 1x, 32(i2) )
            write( 6, 1000 ) ( istruct( ii, ji ), ji = 1, ni )
         end do
      end if

      return
      end
